@@ -16,10 +16,13 @@ def get_solver(rule: AutoRuleBook):
         heuristics=lambda x, y: 0,
         neighbors=rule,
         max_depth=100,
-        silent=False
     )
+    # ahhh we need an identifier for a rulebook.
+    # the identifier needs to be consistent across runs.
+
     cache_path = os.path.expanduser("~/.cache/omni_converter.shelve")
-    return EdgeCachedSolver(solver, cache_path)
+    return EdgeCachedSolver(solver, cache_path)# this is too unstable
+    #return solver
 
 
 class IAutoData(ABC):
@@ -46,12 +49,15 @@ class IAutoData(ABC):
         pass
 
     @abc.abstractmethod
-    def map(self, f) -> "IAutoData":
+    def map(self, f, format=None) -> "IAutoData":
         pass
 
     @abc.abstractmethod
     def override(self, rule: AutoRuleBook) -> "IAutoData":
         pass
+    @abc.abstractmethod
+    def cast(self,new_format)->"IAutoData":
+        pass
 
 
 @dataclass
@@ -73,6 +79,9 @@ class RuledData(IAutoData):
     def override(self, book: AutoRuleBook):
         return RuledData(self.data, book)
 
+    def cast(self,new_format):
+        return AutoData2(self.data.value,new_format).with_rules(self.rulebook)
+
     def converter(self, dst_format):
         conversions = self.solver.solve(self.data.format, dst_format)
         return conversions
@@ -96,11 +105,11 @@ class RuledData(IAutoData):
     def _repr_html_(self):
         return self.to("_repr_html_")
 
-    def map(self, f):
-        mapped, format = f(self.data.value)
+    def map(self, f, format=None):
+        mapped = f(self.data.value)
         nv = AutoData2(
             value=mapped,
-            format=format
+            format=format or self.format
         )
         return RuledData(
             data=nv,
